@@ -21,6 +21,13 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.'''
 
 from __future__ import division
+
+__all__ = ['get_clean_isd_history', 'IntegratedSurfaceDatabaseStation',
+           'get_closest_station', 'get_station_year_text', 'gsod_day_parser',
+           'StationDataGSOD', 'heating_degree_days', 'cooling_degree_days', 'stations',
+#           'geopy_geolocator', 'geopy_cache', 'SimpleGeolocatorCache', 
+           'geocode']
+
 try: # pragma: no cover
     from cStringIO import StringIO
 except: # pragma: no cover
@@ -33,7 +40,7 @@ from collections import namedtuple
 
 import numpy as np
 from fluids.core import F2K
-from scipy.constants import mile, knot, inch
+from fluids.constants import mile, knot, inch
 from scipy.spatial import KDTree, cKDTree
 from scipy.stats import scoreatpercentile
 
@@ -50,17 +57,140 @@ try:  # pragma: no cover
     data_dir = user_config_dir('fluids')
 except ImportError:  # pragma: no cover
     data_dir = ''
-    pass
-# TODO: Import ephem and get hours/minutes of sunlight per day.
+
+try:  # pragma: no cover
+    import geopy
+    from geopy.location import Location
+    # No point loading cPickle or sqlite for this reason
+    import sqlite3
+except ImportError:  # pragma: no cover
+    geopy = None
+    Location = None
+try:  # pragma: no cover
+    # python 3 compat
+    import cPickle as pickle
+except:  # pragma: no cover
+    import pickle
+
+# Geopy cache/lookup layer, also requires appdirs for caching, can work without
+geolocator = None
+geolocator_user_agent = 'fluids'
+geolocator_disk_cache_name = 'simple_geolocator_cache.sqlite3'
+geolocator_disk_cache_loc = os.path.join(data_dir, geolocator_disk_cache_name)
+
+simple_geopy_cache = None
+
+geopy_missing_msg = '''Geocoder module `geopy` is required for this 
+functionality.'''
+
+def geopy_geolocator():
+    '''Lazy loader for geocoder from geopy. This currently loads the 
+    `Nominatim` geocode and returns an instance of it, taking ~2 us.
+    '''
+    global geolocator
+    if geolocator is None:
+        try:
+            from geopy.geocoders import Nominatim
+        except ImportError:
+            return None
+        geolocator = Nominatim(user_agent=geolocator_user_agent)
+        return geolocator
+    return geolocator
+
+
+def geopy_cache():
+    '''Lazy loader for the singleton `SimpleGeolocatorCache`. This creates a
+    sqlite database if one does not exist and initializes a connection to it.
+    '''
+    global simple_geopy_cache
+    if simple_geopy_cache is None:
+        simple_geopy_cache = SimpleGeolocatorCache(geolocator_disk_cache_loc)
+        return simple_geopy_cache
+    return simple_geopy_cache
+
+
+class SimpleGeolocatorCache(object):
+    '''Very basic on-disk address -> (lat, lon) cache, using Python's sqlite 
+    database for on-disk persistence. Offers very reasonable performance 
+    compared to online lookups.
+    '''
+    def __init__(self, file_name):
+        self.connection = conn = sqlite3.connect(file_name)
+        cursor = self.connection.cursor()
+        cursor.execute('CREATE TABLE IF NOT EXISTS geopy ( '
+                       'address STRING PRIMARY KEY, latitude real, longitude real )')
+        self.connection.commit()
+
+    def cached_address(self, address):
+        cursor = self.connection.cursor()
+        cursor.execute('SELECT latitude, longitude FROM geopy WHERE address=?', (address, ))
+        res = cursor.fetchone()
+        if res is None: 
+            return None
+        return res
+
+    def cache_address(self, address, latitude, longitude):
+        cursor = self.connection.cursor()
+        cursor.execute('INSERT INTO geopy(address, latitude, longitude) VALUES(?, ?, ?)',
+                       (address, latitude, longitude))
+        self.connection.commit()
+
+
+def geocode(address):
+    '''Query function to obtain a latitude and longitude from a location
+    string such as `Houston, TX` or`Colombia`. This uses an online lookup,
+    currently wrapping the `geopy` library, and providing an on-disk cache
+    of queries.
+    
+    Parameters
+    ----------
+    address : str
+        Search string to retrieve the location, [-]
+        
+    Returns
+    -------
+    latitude : float
+        Latitude of address, [degrees]
+    longitude : float
+        Longitude of address, [degrees]
+        
+    Notes
+    -----
+    If a query has been retrieved before, this function will take under 1 ms;
+    it takes several seconds otherwise.
+    
+    Examples
+    --------
+    >>> geocode('Fredericton, NB')
+    (45.966425, -66.645813)
+    '''
+    loc_tuple = None
+    try:
+        cache = geopy_cache()
+        loc_tuple = cache.cached_address(address)
+    except:
+        # Handle bugs in the cache, i.e. if there is no space on disk to create
+        # the database, by ignoring them
+        pass
+    if loc_tuple is not None:
+        return loc_tuple
+    else:
+        geocoder = geopy_geolocator()
+        if geocoder is None:
+            return geopy_missing_msg
+        location = geocoder.geocode(address)
+        try:
+            cache.cache_address(address, location.latitude, location.longitude)
+        except:
+            pass
+        return (location.latitude, location.longitude)
+
     
-__all__ = ['get_clean_isd_history', 'IntegratedSurfaceDatabaseStation',
-           'get_closest_station', 'get_station_year_text', 'gsod_day_parser',
-           'StationDataGSOD', 'heating_degree_days', 'cooling_degree_days', 'stations']
 
 folder = os.path.join(os.path.dirname(__file__), 'data')
 
 
-def heating_degree_days(T, T_base=F2K(65), truncate=True):
+def heating_degree_days(T, T_base=291.4833333333333, truncate=True):
     r'''Calculates the heating degree days for a period of time.
 
     .. math::
@@ -205,6 +335,13 @@ class IntegratedSurfaceDatabaseStation(object):
     
     License information for the database can be found at the following link:
     https://data.noaa.gov/dataset/global-surface-summary-of-the-day-gsod
+    
+    Note: Of the 28000 + stations in the database, approximately 3000 have WBAN
+    identifiers; 26000 have unique names; 24000 have USAF identifiers; and 
+    there are only 25800 unique lat/lon pairs.
+    
+    To uniquely represent a weather station, a combination of identifiers 
+    must be used. (Name, USAF, WBAN) makes a good choice.
 
     Parameters
     ----------
@@ -342,7 +479,7 @@ class StationDataGSOD(object):
             for i, day in enumerate(data):
                 if day is None:
                     continue
-                # Don't do these comparisions to make it fast
+                # Don't do these comparisons to make it fast
                 if day.DATE.year < older_year or day.DATE.year > newer_year:
                     continue # Ignore out-of-range days as possible
                     
@@ -406,7 +543,7 @@ class StationDataGSOD(object):
             for i, day in enumerate(data):
                 if day is None:
                     continue
-                # Don't do these comparisions to make it fast
+                # Don't do these comparisons to make it fast
                 if day.DATE.year < older_year or day.DATE.year > newer_year:
                     continue # Ignore out-of-range days as possible
                     
@@ -558,10 +695,10 @@ def get_closest_station(latitude, longitude, minumum_recent_data=20140000,
                     'specified near the specified coordinates.')
 
 
-# This should be agressively cached
+# This should be aggressively cached
 def get_station_year_text(WMO, WBAN, year):
     '''Basic method to download data from the GSOD database, given a 
-    station idenfifier and year. 
+    station identifier and year. 
 
     Parameters
     ----------
