@@ -22,8 +22,10 @@ SOFTWARE.'''
 
 from fluids import *
 import numpy as np
-from scipy.constants import inch
+from fluids.constants import inch
 from scipy.optimize import fsolve
+from math import log10
+from fluids.numerics import linspace, logspace, assert_close, isclose
 from numpy.testing import assert_allclose
 import pytest
 
@@ -54,7 +56,108 @@ def test_C_Reader_Harris_Gallagher():
 #def test_Reader_Harris_Gallagher_discharge():
 #    m = Reader_Harris_Gallagher_discharge(D=0.07366, Do=0.05, P1=200000.0, P2=183000.0, rho=999.1, mu=0.0011, k=1.33, taps='D')
 #    assert_allclose(m, 7.702338035732167)
+
+    with pytest.raises(Exception):
+        C_Reader_Harris_Gallagher(D=0.07391, Do=0.0222, rho=1.165, mu=1.85E-5,  m=0.12, taps='NOTALOCATION') 
     
+    # Test continuity at the low-diameter function
+    kwargs = dict(Do=0.0222, rho=1.1645909036, mu=0.0000185861753095, m=0.124431876, taps='corner')
+    C1 = C_Reader_Harris_Gallagher(D=0.07112, **kwargs)
+    C2 = C_Reader_Harris_Gallagher(D=0.07112-1e-13, **kwargs)
+    assert_allclose(C1, C2)
+    
+def test_C_Miller_1996():
+    C_flange_ISO = C_Reader_Harris_Gallagher(D=0.07391, Do=0.0222, rho=1.165, mu=1.85E-5, m=0.12, taps='flange')
+    C_corner_ISO = C_Reader_Harris_Gallagher(D=0.07391, Do=0.0222, rho=1.165, mu=1.85E-5, m=0.12, taps='corner')
+    C_D_D2_ISO = C_Reader_Harris_Gallagher(D=0.07391, Do=0.0222, rho=1.165, mu=1.85E-5, m=0.12, taps=ORIFICE_D_AND_D_2_TAPS)
+    
+    C_flange = C_Miller_1996(D=0.07391, Do=0.0222, rho=1.165, mu=1.85E-5, m=0.12, subtype=MILLER_ORIFICE, taps=ORIFICE_FLANGE_TAPS)
+    C_flange_2 = C_Miller_1996(D=0.07391, Do=0.0222, rho=1.165, mu=1.85E-5, m=0.12, subtype='orifice', taps=ORIFICE_FLANGE_TAPS)
+    assert C_flange == C_flange_2
+    assert_close(C_flange, 0.599065557156788, rtol=1e-12)
+    assert_close(C_flange, C_flange_ISO, rtol=2e-4)
+    
+    C_flange_small_ISO = C_Reader_Harris_Gallagher(D=0.04, Do=0.02, rho=1.165, mu=1.85E-5, m=0.2, taps='flange')
+    C_flange_small = C_Miller_1996(D=0.04, Do=0.02, rho=1.165, mu=1.85E-5, m=0.2, subtype=MILLER_ORIFICE, taps=ORIFICE_FLANGE_TAPS)
+    assert_close(C_flange_small, 0.6035249226284967, rtol=1e-12)
+    assert_close(C_flange_small_ISO, C_flange_small, rtol=1e-2)
+    
+    C_corner = C_Miller_1996(D=0.07391, Do=0.0222, rho=1.165, mu=1.85E-5, m=0.12, subtype=MILLER_ORIFICE, taps=ORIFICE_CORNER_TAPS)
+    assert_close(C_corner, 0.5991255880475622, rtol=1e-12)
+    assert_close(C_corner, C_corner_ISO, rtol=2e-3)
+    
+    C_D_D2 = C_Miller_1996(D=0.07391, Do=0.0222, rho=1.165, mu=1.85E-5, m=0.12, subtype=MILLER_ORIFICE, taps=ORIFICE_D_AND_D_2_TAPS)
+    assert_close(C_D_D2, 0.5836056345693277, rtol=1e-12)
+    assert_close(C_D_D2, C_D_D2_ISO, rtol=3e-2)
+    
+    C_pipe = C_Miller_1996(D=0.07391, Do=0.0222, rho=1.165, mu=1.85E-5, m=0.12, subtype=MILLER_ORIFICE, taps=ORIFICE_PIPE_TAPS)
+    assert_close(C_pipe, 0.6338716097225481, rtol=1e-12)
+    
+    
+    
+    C_flange_small = C_Miller_1996(D=0.07391, Do=0.0222, rho=1.165, mu=1.85E-5, m=0.12, subtype=MILLER_SEGMENTAL_ORIFICE, taps=ORIFICE_FLANGE_TAPS)
+    C_flange_small2 = C_Miller_1996(D=0.07391, Do=0.0222, rho=1.165, mu=1.85E-5, m=0.12, subtype='segmental orifice', taps=ORIFICE_FLANGE_TAPS)
+    assert C_flange_small == C_flange_small
+    
+    C_flange_large = C_Miller_1996(D=0.2, Do=0.08, rho=1.165, mu=1.85E-5, m=2, subtype=MILLER_SEGMENTAL_ORIFICE, taps=ORIFICE_FLANGE_TAPS)
+    assert_close(C_flange_small, 0.6343546437000684, rtol=1e-12)
+    assert_close(C_flange_large, 0.6301688962913937, rtol=1e-12)
+    
+    C_vc_small = C_Miller_1996(D=0.07391, Do=0.0222, rho=1.165, mu=1.85E-5, m=0.12, subtype=MILLER_SEGMENTAL_ORIFICE, taps=ORIFICE_VENA_CONTRACTA_TAPS)
+    C_vc_large = C_Miller_1996(D=0.2, Do=0.08, rho=1.165, mu=1.85E-5, m=2, subtype=MILLER_SEGMENTAL_ORIFICE, taps=ORIFICE_VENA_CONTRACTA_TAPS)
+    assert_close(C_vc_small, 0.6341386019820933, rtol=1e-12)
+    assert_close(C_vc_large, 0.6301688962913937, rtol=1e-12)
+    
+    C_flange_opp_small = C_Miller_1996(D=0.07391, Do=0.0222, rho=1.165, mu=1.85E-5, m=0.12, subtype=MILLER_ECCENTRIC_ORIFICE, taps='flange', tap_position=TAPS_OPPOSITE)
+    C_flange_opp_small2 = C_Miller_1996(D=0.07391, Do=0.0222, rho=1.165, mu=1.85E-5, m=0.12, subtype='eccentric orifice', taps='flange', tap_position=TAPS_OPPOSITE)
+    assert_close(C_flange_opp_small, 0.6096299230744815, rtol=1e-12)
+    C_flange_opp_large = C_Miller_1996(D=0.2, Do=0.08, rho=1.165, mu=1.85E-5, m=2, subtype=MILLER_ECCENTRIC_ORIFICE, taps='flange', tap_position=TAPS_OPPOSITE)
+    assert_close(C_flange_opp_large, 0.6196903510975135, rtol=1e-12)
+    
+    C_flange_side_small = C_Miller_1996(D=0.07391, Do=0.0222, rho=1.165, mu=1.85E-5, m=0.12, subtype=MILLER_ECCENTRIC_ORIFICE, taps='flange', tap_position=TAPS_SIDE)
+    C_flange_side_large = C_Miller_1996(D=0.2, Do=0.08, rho=1.165, mu=1.85E-5, m=2, subtype=MILLER_ECCENTRIC_ORIFICE, taps='flange', tap_position=TAPS_SIDE)
+    assert_close(C_flange_side_small, 0.6086231594104639, rtol=1e-12)
+    assert_close(C_flange_side_large, 0.6227796822413327, rtol=1e-12)
+    
+    
+    C_vc_opp_small = C_Miller_1996(D=0.07391, Do=0.0222, rho=1.165, mu=1.85E-5, m=0.12, subtype=MILLER_ECCENTRIC_ORIFICE, taps=ORIFICE_VENA_CONTRACTA_TAPS, tap_position=TAPS_OPPOSITE)
+    assert_close(C_vc_opp_small, 0.6108105171632562, rtol=1e-12)
+    C_vc_opp_large = C_Miller_1996(D=0.2, Do=0.08, rho=1.165, mu=1.85E-5, m=2, subtype=MILLER_ECCENTRIC_ORIFICE, taps=ORIFICE_VENA_CONTRACTA_TAPS, tap_position=TAPS_OPPOSITE)
+    assert_close(C_vc_opp_large, 0.6190713098741648, rtol=1e-12)
+    
+    C_vc_side_small = C_Miller_1996(D=0.07391, Do=0.0222, rho=1.165, mu=1.85E-5, m=0.12, subtype=MILLER_ECCENTRIC_ORIFICE, taps=ORIFICE_VENA_CONTRACTA_TAPS, tap_position=TAPS_SIDE)
+    C_vc_side_large = C_Miller_1996(D=0.2, Do=0.08, rho=1.165, mu=1.85E-5, m=2, subtype=MILLER_ECCENTRIC_ORIFICE, taps=ORIFICE_VENA_CONTRACTA_TAPS, tap_position=TAPS_SIDE)
+    assert_close(C_vc_side_small, 0.6089351556538237, rtol=1e-12)
+    assert_close(C_vc_side_large, 0.6214809940486437, rtol=1e-12)
+    
+    # Error testing
+    with pytest.raises(ValueError):
+        C_Miller_1996(D=0.07391, Do=0.0222, rho=1.165, mu=1.85E-5, m=0.12, subtype=MILLER_ORIFICE, taps='NOTATAP')
+
+    with pytest.raises(ValueError):
+        C_Miller_1996(D=0.07391, Do=0.0222, rho=1.165, mu=1.85E-5, m=0.12, subtype=MILLER_ECCENTRIC_ORIFICE, taps='NOTATAP')
+    
+    with pytest.raises(ValueError):
+        C_Miller_1996(D=0.07391, Do=0.0222, rho=1.165, mu=1.85E-5, m=0.12, subtype=MILLER_ECCENTRIC_ORIFICE, taps=ORIFICE_FLANGE_TAPS, tap_position='NOTAPOSITION')
+
+    with pytest.raises(ValueError):
+        C_Miller_1996(D=0.2, Do=0.08, rho=1.165, mu=1.85E-5, m=2, subtype=MILLER_SEGMENTAL_ORIFICE, taps='BADTAP')
+        
+    with pytest.raises(ValueError):
+        C_Miller_1996(D=0.07391, Do=0.0222, rho=1.165, mu=1.85E-5, m=0.12, subtype='BADTYPE')
+    
+    # Conical
+    C_high = C_Miller_1996(D=0.07391, Do=0.0222, rho=1.165, mu=1.85E-5, m=0.12, subtype=MILLER_CONICAL_ORIFICE)
+    assert C_high == 0.73
+    C_low = C_Miller_1996(D=0.07391, Do=0.0222, rho=1.165, mu=1.85E-5, m=0.0001, subtype=MILLER_CONICAL_ORIFICE)
+    assert C_low == 0.734
+    C_low2 = C_Miller_1996(D=0.07391, Do=0.0222, rho=1.165, mu=1.85E-5, m=0.0001, subtype='conical orifice')
+    assert C_low2 == C_low
+    # Quarter circle
+    C_circ = C_Miller_1996(D=0.07391, Do=0.0222, rho=1.165, mu=1.85E-5, m=0.12, subtype=MILLER_QUARTER_CIRCLE_ORIFICE)
+    assert_close(C_circ, 0.7750496225919683)
+    C_circ2 = C_Miller_1996(D=0.07391, Do=0.0222, rho=1.165, mu=1.85E-5, m=0.12, subtype='quarter circle orifice')
+    assert C_circ == C_circ2
     
 def test_differential_pressure_meter_discharge():
     # Orifice
@@ -88,6 +191,9 @@ def test_differential_pressure_meter_discharge():
     # wedge meter
     m = differential_pressure_meter_solver(D=0.07366, D2=0.05, P1=200000.0, P2=183000.0, rho=999.1, mu=0.0011, k=1.33, meter_type=WEDGE_METER)
     assert_allclose(m, 8.941980099523539)
+    
+    with pytest.raises(ValueError):
+        differential_pressure_meter_solver(D=.07366, m=7.702338, P2=183000.0, rho=999.1, mu=0.0011, k=1.33, meter_type='ISO 5167 orifice', taps='D')
 
 
 def test_differential_pressure_meter_diameter():
@@ -189,6 +295,45 @@ def test_differential_pressure_meter_P1():
     assert_allclose(P1, 200000)
 
 
+
+
+def test_differential_pressure_meter_solver_limits():
+    # ISO 5167 orifice - How low can P out go?
+    P_out = differential_pressure_meter_solver(D=0.07366, m=7.702338, P1=200000.0, D2=0.0345, rho=999.1, mu=0.0011, k=1.33, meter_type='ISO 5167 orifice', taps='D')
+    assert_close(P_out, 37914.15989971644)
+    
+    # same point
+    D2_recalc = differential_pressure_meter_solver(D=0.07366, m=7.702338, P1=200000.0, P2=37914.15989971644, rho=999.1, mu=0.0011, k=1.33, meter_type='ISO 5167 orifice', taps='D')
+    assert_close(D2_recalc, 0.0345)
+    
+    P1_recalc = differential_pressure_meter_solver(D=0.07366, m=7.702338, P2=37914.15989971644, D2=0.0345, rho=999.1, mu=0.0011, k=1.33, meter_type='ISO 5167 orifice', taps='D')
+    assert_close(P1_recalc, 200000.0)
+    
+    m_recalc = differential_pressure_meter_solver(D=0.07366, P1=200000, P2=37914.15989971644, D2=0.0345, rho=999.1, mu=0.0011, k=1.33, meter_type='ISO 5167 orifice', taps='D')
+    assert_close(m_recalc, 7.702338)
+    
+def test_C_eccentric_orifice_ISO_15377_1998():
+    C =  C_eccentric_orifice_ISO_15377_1998(.2, .075)
+    assert_close(C, 0.6351923828125)
+    
+    # Does not perfectly match - like error in ISO.
+    D = 1.0
+    betas = [1e-2*i for i in range(46, 85, 1)]
+    Cs_expect = [0.627, 0.627, 0.627, 0.627, 0.627, 0.627, 0.627, 0.627, 0.627, 0.628, 0.628, 0.628, 0.628, 0.629, 0.629, 0.629, 0.629, 0.629, 0.629, 0.629, 0.629, 0.629, 0.628, 0.628, 0.627, 0.626, 0.625, 0.624, 0.623, 0.621, 0.620, 0.618, 0.616, 0.613, 0.611, 0.608, 0.605, 0.601, 0.597]
+    Cs_calc = [C_eccentric_orifice_ISO_15377_1998(D=D, Do=beta_i) for beta_i in betas]
+    for Ci, Cj in zip(Cs_expect, Cs_calc):
+        assert isclose(Ci, Cj, rel_tol=1.02e-3)
+    
+def test_C_quarter_circle_orifice_ISO_15377_1998():
+    C = C_quarter_circle_orifice_ISO_15377_1998(.2, .075)
+    assert_close(C, 0.7785148437500001, rtol=1e-12)
+    
+    betas = [0.245, 0.250, 0.260, 0.270, 0.280, 0.290, 0.300, 0.310, 0.320, 0.330, 0.340, 0.350, 0.360, 0.370, 0.380, 0.390, 0.400, 0.410, 0.420, 0.430, 0.440, 0.450, 0.460, 0.470, 0.480, 0.490, 0.500, 0.510, 0.520, 0.530, 0.540, 0.550, 0.560, 0.570, 0.580, 0.590, 0.600]
+    Cs_expect = [0.772, 0.772, 0.772, 0.773, 0.773, 0.773, 0.774, 0.774, 0.775, 0.775, 0.776, 0.776, 0.777, 0.778, 0.779, 0.780, 0.781, 0.783, 0.784, 0.786, 0.787, 0.789, 0.791, 0.794, 0.796, 0.799, 0.802, 0.805, 0.808, 0.812, 0.816, 0.820, 0.824, 0.829, 0.834, 0.839, 0.844]
+    for Do, C_expect in zip(betas, Cs_expect):
+        C = C_quarter_circle_orifice_ISO_15377_1998(D=1, Do=Do)
+        assert (round(C, 3) == C_expect)
+    
 def test_K_to_discharge_coefficient():
     C = K_to_discharge_coefficient(D=0.07366, Do=0.05, K=5.2314291729754)
     assert_allclose(C, 0.6151200000000001)
@@ -282,10 +427,19 @@ def test_dP_cone_meter():
 
 
 def test_C_wedge_meter_Miller():
+    # Large bore
     D = 0.15239999999999998
     C = C_wedge_meter_Miller(D=D, H=0.3*D)
     assert_allclose(C, 0.7267069372687651)
     
+    # Tiny bore
+    C = C_wedge_meter_Miller(D=.6*inch, H=0.3*.6*inch)
+    assert_allclose(C, 0.8683022107124251)
+    
+    # Medium bore
+    C = C_wedge_meter_Miller(D=1.3*inch, H=0.3*1.3*inch)
+    assert_allclose(C, 1.15113726440674)
+    
     
 def test_C_wedge_meter_ISO_5167_6_2017():
     C = C_wedge_meter_ISO_5167_6_2017(D=0.1524, H=0.3*0.1524)
@@ -340,6 +494,27 @@ def test_differential_pressure_meter_dP():
 
     dP = differential_pressure_meter_dP(D=0.07366, D2=0.05, P1=200000.0, P2=183000.0,  meter_type=WEDGE_METER)
     assert_allclose(dP, 7112.927753356824)
+    
+    with pytest.raises(Exception):
+        differential_pressure_meter_dP(D=0.07366, D2=0.05, P1=200000.0,  P2=183000.0, meter_type=VENTURI_NOZZLE) 
+
+    with pytest.raises(ValueError):
+        differential_pressure_meter_dP(D=0.07366, D2=0.05, P1=200000.0, P2=183000.0, meter_type='NOTAMETER')
+
+
+
+def test_differential_pressure_meter_beta():
+    beta = differential_pressure_meter_beta(D=0.2575, D2=0.184, meter_type=LONG_RADIUS_NOZZLE)
+    assert_allclose(beta, 0.7145631067961165)
+    
+    beta = differential_pressure_meter_beta(D=0.2575, D2=0.184, meter_type=WEDGE_METER)
+    assert_allclose(beta, 0.8743896375172885)
+    
+    beta = differential_pressure_meter_beta(D=0.2575, D2=0.184, meter_type=CONE_METER)
+    assert_allclose(beta, 0.6995709873957624)
+    
+    with pytest.raises(ValueError):
+        differential_pressure_meter_beta(D=0.07366, D2=0.05, meter_type='NOTAMETER')
 
 
 def test_cone_meter_expansibility_Stewart_full():
@@ -507,8 +682,58 @@ def test_C_venturi_nozzle_full():
     Cs_calc = [C_venturi_nozzle(D=1, Do=beta) for beta in betas]
     assert_allclose(Cs, Cs_calc, rtol=5E-3)
 
+def test_differential_pressure_meter_C_epsilon():
+    # Some random cases
+    C, eps = differential_pressure_meter_C_epsilon(D=0.07366, D2=0.05, P1=200000.0, 
+    P2=183000.0, rho=999.1, mu=0.0011, k=1.33, m=7.702338035732168,
+    meter_type=ISO_15377_ECCENTRIC_ORIFICE)
+    assert_close(C, 0.6284616939680627)
+    assert_close(eps, 0.9711026966676307)
+    
+    C, eps = differential_pressure_meter_C_epsilon(D=0.07366, D2=0.05, P1=200000.0, 
+    P2=183000.0, rho=999.1, mu=0.0011, k=1.33, m=7.702338035732168,
+    meter_type=ISO_15377_QUARTER_CIRCLE_ORIFICE)
+    assert_close(C, 0.899402420975695)
+    assert_close(eps, 0.9711026966676307)
+    
+    C, eps = differential_pressure_meter_C_epsilon(D=0.07366, D2=0.05, P1=200000.0, 
+    P2=183000.0, rho=999.1, mu=0.0011, k=1.33, m=7.702338035732168,
+    meter_type=ISO_15377_CONICAL_ORIFICE)
+    assert_close(C, 0.734)
+    assert_close(eps, 0.9532330165749132)
+    
+    C, eps = differential_pressure_meter_C_epsilon(D=0.07366, D2=0.05, P1=200000.0, 
+    P2=183000.0, rho=999.1, mu=0.0011, k=1.33, m=7.702338035732168,
+    meter_type=MILLER_ORIFICE, taps='corner')
+    assert_close(C, 0.6068011224659587)
+    assert_close(eps, 0.9711026966676307)
+    
+    C, eps = differential_pressure_meter_C_epsilon(D=0.07366, D2=0.05, P1=200000.0, 
+    P2=183000.0, rho=999.1, mu=0.0011, k=1.33, m=7.702338035732168,
+    meter_type=MILLER_CONICAL_ORIFICE)
+    assert_close(C, 0.73)
+    assert_close(eps, 0.9532330165749132)
+    
+    
+    # Test one case of the default translation
+    C, eps = differential_pressure_meter_C_epsilon(D=0.07366, D2=0.05, P1=200000.0, 
+    P2=183000.0, rho=999.1, mu=0.0011, k=1.33, m=7.702338035732168,taps='corner',
+    meter_type=CONCENTRIC_ORIFICE)
+    C_iso, eps_iso = differential_pressure_meter_C_epsilon(D=0.07366, D2=0.05, P1=200000.0, 
+    P2=183000.0, rho=999.1, mu=0.0011, k=1.33, m=7.702338035732168,taps='corner',
+    meter_type=CONCENTRIC_ORIFICE)
+    
+    assert C == C_iso
+    assert eps == eps_iso
+    
+    with pytest.raises(ValueError):
+        differential_pressure_meter_C_epsilon(D=0.07366, D2=0.05, P1=200000.0, 
+                                              P2=183000.0, rho=999.1, mu=0.0011, 
+                                              k=1.33, m=7.702338035732168, meter_type='NOTAREAMETER')
+        
+    
 
-
+@pytest.mark.fuzz
 @pytest.mark.slow
 def test_fuzz_K_to_discharge_coefficient():
     '''
@@ -533,12 +758,12 @@ def test_fuzz_K_to_discharge_coefficient():
     print(latex(solve(expr, C)[3]))
     '''
     
-    Ds = np.logspace(np.log10(1-1E-9), np.log10(1E-9))
+    Ds = logspace(log10(1-1E-9), log10(1E-9), 8)
     for D_ratio in Ds:
-        Ks = np.logspace(np.log10(1E-9), np.log10(50000))
+        Ks = logspace(log10(1E-9), log10(50000), 8)
         Ks_recalc = []
         for K in Ks:
-            C = K_to_discharge_coefficient(D=1, Do=D_ratio, K=K)
-            K_calc = discharge_coefficient_to_K(D=1, Do=D_ratio, C=C)
+            C = K_to_discharge_coefficient(D=1.0, Do=D_ratio, K=K)
+            K_calc = discharge_coefficient_to_K(D=1.0, Do=D_ratio, C=C)
             Ks_recalc.append(K_calc)
         assert_allclose(Ks, Ks_recalc)
\ No newline at end of file
